#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (c) 2008 Oracle Inc.
# Copyright (c) 2008 Zhigang Wang <zhigang.x.wang@oracle.com>
#============================================================================

from types import StringTypes

import xen.lowlevel.xc
from xen.xend.XendLogging import log
from xen.xend.XendBase import XendBase
from xen.xend.xenstore.xstransact import xstransact
from xen.xend.xenstore.xsutil import GetDomainPath

xc = xen.lowlevel.xc.xc()

class XendVMGuestMetrics(XendBase):
    """VM Guest Metrics."""

    def getClass(self):
        return "VM_guest_metrics"
    
    def getAttrRO(self):
        attrRO = ['os_version',
                  'PV_drivers_version',
                  'memory',
                  'disks',
                  'networks',
                  'other',
                  'last_updated']
        return XendBase.getAttrRO() + attrRO

    def getMethods(self):
        methods = ['update']
        return XendBase.getMethods() + methods

    getClass    = classmethod(getClass)
    getAttrRO   = classmethod(getAttrRO)
    getMethods  = classmethod(getMethods)

    def __init__(self, uuid, xend_domain_instance):
        XendBase.__init__(self, uuid, {})
        self.xend_domain_instance = xend_domain_instance

    def comma_sep_kv_to_dict(self, c):
        """Convert comma-separated, equals-separated key-value pairs into a
        dictionary.
        """
        d = {}
        if type(c) in StringTypes:
            c = c.strip()
            if len(c) > 0:
                a = c.split(',')
                for b in a:
                    if b.find('=') == -1:
                        log.exception("%s should be a pair, separated by an equals sign." % b)
                        break
                    (k, v) = b.split('=', 1)
                    k = k.strip()
                    v = v.strip()
                    d[k] = v
        return d

    def _readKey(self, key):
        if self.xend_domain_instance.getDomid():
            return xstransact.Read('%s/guest/metrics/%s' % (self.xend_domain_instance.getDomainPath(), key))

    def _listKey(self, key):
        if self.xend_domain_instance.getDomid():
            return xstransact.List('%s/guest/metrics/%s' % (self.xend_domain_instance.getDomainPath(), key))
        else:
            return []

    def get_os_version(self):
        return self.comma_sep_kv_to_dict(self._readKey('os_version'))

    def get_PV_drivers_version(self):
        return self.comma_sep_kv_to_dict(self._readKey('PV_drivers_version'))
    
    def get_memory(self):
        return self.comma_sep_kv_to_dict(self._readKey('memory'))

    def get_other(self):
        return self.comma_sep_kv_to_dict(self._readKey('other'))

    def get_disks(self):
        disks = {}
        for num in self._listKey('disks'):
            disks[num] = self._readKey('disks/%s' % num)

        return disks

    def get_networks(self):
        networks = {}
        for num in self._listKey('networks'):
            networks[num] = self._readKey('networks/%s' % num)

        return networks

    def get_last_updated(self):
        value = self._readKey('last_updated')
        import xen.xend.XendAPI as XendAPI
        if value:
            return XendAPI.datetime(float(value))
        else:
            return XendAPI.datetime(0.0)

    def update(self):
        """Ask the guest tools to update the metrics."""
        if self.xend_domain_instance.getDomid():
            xstransact.Write('%s/guest' % self.xend_domain_instance.getDomainPath(), 'evtchn', 'UPDATE')

